
%option c++
%option yyclass="SatLexer"
%option noyywrap
%option yylineno
%option case-insensitive
%option full
%option prefix="SAT_"

%{
    #include <iostream>
    #include <sstream>
    #include "parser.hpp"
    #include "satlexer.h"
%}
%% // --------------------------------------------------------------------------
<<EOF>>                                 {return yy::SatParser::token::TOK_EOF;}
"#"[^\n]*\n                             { /* comment */ }
"//"[^\n]*\n                            { /* comment */ }
[\r\n\t\ ]*                             { /* ignore whitespace */ }
";"                                     {return yy::SatParser::token::SEMICOLON;}
"true"                                  {return yy::SatParser::token::TRUE;}
"false"                                 {return yy::SatParser::token::FALSE;}
"&"|"&&"                                {return yy::SatParser::token::AND;}
"|"|"||"                                {return yy::SatParser::token::OR;}
"~"|"!"                                 {return yy::SatParser::token::NOT;}
"->"                                    {return yy::SatParser::token::IMPL;}
"<->"|"=="                              {return yy::SatParser::token::IFF;}
"("                                     {return yy::SatParser::token::OPEN;}
")"                                     {return yy::SatParser::token::CLOSE;}
[a-zA-Z][a-zA-Z0-9]*                    {
                                          yylval->stringval = new std::string(yytext);
                                          return yy::SatParser::token::IDENTIFIER;
                                        }
.                                       {
                                          std::cerr << "\n\nLEXICAL ERROR (Line "
                                            << yylineno << "): Unexpected character \""
                                            << yytext[0] << "\"\n";
                                        }
%% //---------------------------------------------------------------------------

SatLexer::SatLexer(std::istream& is)
    : SAT_FlexLexer(&is)
{
}

int SatLexer::yylex(yy::SatParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}

int SatLexer::GetCurrentLineNumber()
{
    return yylineno;
}
