%code requires {

  #define YYERROR_VERBOSE
  #include <iostream>
  #include "sat.h"
  class SatLexer; // forward declaration

} // Bison Declarations

%defines
%define api.parser.class {SatParser}
%define parse.error detailed
%parse-param { SatLexer &lexer }
%parse-param { std::vector<Sat::SatFormula*>* result }
%lex-param { SatLexer &lexer }

%union
{
	std::string* stringval;
	Sat::SatFormula* formula;
	std::vector<Sat::SatFormula*>* formulas;
};

%skeleton "lalr1.cc"
%start S

%token SEMICOLON TRUE FALSE AND OR NOT IMPL IFF OPEN CLOSE
%token TOK_EOF 0 "end of file"
%token <stringval> IDENTIFIER

%type <formulas> formulas
%type <formula> SAT
%type <formula> SAT_A
%type <formula> SAT_B
%type <formula> SAT_C
%type <formula> SAT_D

%code {
	static int yylex(yy::SatParser::semantic_type* yylval, SatLexer &scanner);
}

%% // Produktionen
S
	: formulas TOK_EOF
	{
		for(Sat::SatFormula* formula : *$formulas)
			result->push_back(formula);
		delete $formulas;
	}
;

formulas
	: %empty
	{
		$$ = new std::vector<Sat::SatFormula*>();
	}
	| formulas[fs] SAT[f] SEMICOLON
	{
		$$ = $fs;
		$$->push_back($f);
	}
;

SAT
	: SAT_A
	| SAT_A[l] IFF SAT_A[r]
	{
		$$ = new Sat::SatIff($l, $r);
	}
;
SAT_A
	: SAT_B
	| SAT_A[l] OR SAT_B[r]
	{
		$$ = new Sat::SatOr($l, $r);
	}
;

SAT_B
	: SAT_C
	| SAT_B[l] AND SAT_C[r]
	{
		$$ = new Sat::SatAnd($l, $r);
	}
;

SAT_C
	: SAT_D
	| NOT SAT_D[f]
	{
		$$ = new Sat::SatNot($f);
	}
;

SAT_D
	: TRUE
	{
		$$ = new Sat::SatTrue();
	}
	| FALSE
	{
		$$ = new Sat::SatFalse();
	}
	| IDENTIFIER
	{
		$$ = new Sat::SatVariable(*$IDENTIFIER);
		delete $IDENTIFIER;
	}
	| OPEN SAT[f] CLOSE
	{
		$$ = $f;
	}
;

%%

#include "satlexer.h"

void yy::SatParser::error(const std::string& s)
{
	std::cerr << "SYNTACTIC ERROR (Line " << lexer.GetCurrentLineNumber() << "): " << s << std::endl;
}

static int yylex(yy::SatParser::semantic_type* yylval, SatLexer &lexer)
{
	return lexer.yylex(yylval);
}
