
#ifndef __SAT_H
#define __SAT_H


#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <set>

namespace Cnf { class CnfFormula; class CnfVariable; }

namespace Sat
{

	class SatFormula;

	class SatAssignment
	{
		public:
			std::map<std::string, bool> values;
			bool Solve(std::vector<SatFormula*>& formulas);
			bool Solve(std::vector<SatFormula*>& formulas, std::map<std::string, bool>::iterator v);
	};

	class SatFormula
	{
		public:
			static int nextId;
			int id;
			SatFormula();
			virtual ~SatFormula();
			virtual void CollectVariables(std::set<std::string>* variables);
			virtual void Write(std::ostream& os) = 0;
			virtual bool Evaluate(SatAssignment* assignment) = 0;

			virtual Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) = 0;
			virtual std::vector<Sat::SatFormula*> CollectConjunctions();
			virtual std::vector<Sat::SatFormula*> CollectDisjunctions();
			Cnf::CnfFormula* ToCnf();
	};

	class SatTrue : public SatFormula
	{
		public:
			SatTrue();
			~SatTrue();
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;
	};

	class SatFalse : public SatFormula
	{
		public:
			SatFalse();
			~SatFalse();
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;

	};

	class SatVariable : public SatFormula
	{
		public:
			std::string name;

			SatVariable(std::string name);
			~SatVariable();
			void CollectVariables(std::set<std::string>* variables) override;
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;
	};

	class SatAnd : public SatFormula
	{
		public:
			SatFormula* l;
			SatFormula* r;

			SatAnd(SatFormula* l, SatFormula* r);
			~SatAnd();
			void CollectVariables(std::set<std::string>* variables) override;
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;
			virtual std::vector<Sat::SatFormula*> CollectConjunctions();
	};

	class SatOr : public SatFormula
	{
		public:
			SatFormula* l;
			SatFormula* r;

			SatOr(SatFormula* l, SatFormula* r);
			~SatOr();
			void CollectVariables(std::set<std::string>* variables) override;
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;
			virtual std::vector<Sat::SatFormula*> CollectDisjunctions();
	};

	class SatNot : public SatFormula
	{
		public:
			SatFormula* f;

			SatNot(SatFormula* f);
			~SatNot();
			void CollectVariables(std::set<std::string>* variables) override;
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;

	};

	class SatIff : public SatFormula
	{
		public:
			SatFormula* l;
			SatFormula* r;

			SatIff(SatFormula* l, SatFormula* r);
			~SatIff();
			void CollectVariables(std::set<std::string>* variables) override;
			bool Evaluate(SatAssignment* assignment) override;
			void Write(std::ostream& os) override;
			Cnf::CnfVariable* ToCnf(Cnf::CnfFormula* formula) override;

	};

}

#endif
